@@ -1,13 +1,17 @@
-#pylint: disable=missing-module-docstring, missing-function-docstring, line-too-long, missing-class-docstring, broad-except
 import subprocess
 
+
 class FileConvert:
 
     def __init__(self, convert_binary: str, logger) -> None:
         self.log = logger
         self.convert_binary = convert_binary
 
-    def convert_file(self, filename, convert_params, message=None, convert_text_parameter: str=None):
+    def convert_file(self,
+                     filename,
+                     convert_params,
+                     message=None,
+                     convert_text_parameter: str = None):
         try:
             self.log.trace(f'convert_file {filename}')
 
@@ -23,11 +27,13 @@ class FileConvert:
                 *text,
                 f'{filename}',
                 f'{filename}'
-                ]
+            ]
 
-            result = subprocess.run(params, capture_output=True, text=True, check=True)
+            result = subprocess.run(
+                params, capture_output=True, text=True, check=True)
             self.log.trace(result.stdout)
             self.log.trace(result.stderr)
+        #pylint: disable=broad-except
         except Exception as error:
             self.log.error(error)
-#pylint: enable=missing-module-docstring, missing-function-docstring, line-too-long, missing-class-docstring, broad-except
+        #pylint: enable=broad-except
